<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2006-2011 Franz Holzinger <franz@ttproducts.de>
*  All rights reserved
*
*  This script is part of the Typo3 project. The Typo3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*  A copy is found in the textfile GPL.txt and important notices to the license
*  from the author is found in LICENSE.txt distributed with these scripts.
*
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * Part of the tt_products (Shop System) extension.
 *
 * class with functions to control all activities
 *
 * $Id: class.tx_ttproducts_control.php 48341 2011-06-02 06:55:07Z franzholz $
 *
 * @author  Franz Holzinger <franz@ttproducts.de>
 * @package TYPO3
 * @subpackage tt_products
 *
 *
 */

global $TYPO3_CONF_VARS;

require_once (PATH_BE_ttproducts.'model/class.tx_ttproducts_model_activity.php');
require_once (PATH_BE_ttproducts.'lib/class.tx_ttproducts_info.php');


class tx_ttproducts_control {
	var $pibase; // reference to object of pibase
	var $cnf;
	var $conf;
	var $config;
	var $basket; 	// the basket object

	var $tt_products; 			// object of the type tx_ttproducts_product
	var $tt_products_articles;		// object of the type tx_table_db
	var $tt_products_cat;			// object of the type tx_ttproducts_category
	var $fe_users;				// object of the type tx_table_db
	var $paymentshipping; 			// object of the type tx_ttproducts_paymentshipping
	var $templateCode='';			// In init(), set to the content of the templateFile. Used by default in getView()
	var $activityArray;			// activities for the CODEs
	var $viewTable;
	var $price;
	var $error_code = array();
	var $marker; // marker functions


	/**
	 * Getting all tt_products_cat categories into internal array
	 */
	function init (&$pibase, &$cnf, &$templateCode, &$basket, &$tt_products, &$tt_products_articles, &$tt_products_cat, &$fe_users, &$price, &$paymentshipping, &$error_code)  {
		global $TYPO3_DB,$TSFE,$TCA;

		$this->pibase = &$pibase;
		$this->cnf = &$cnf;
		$this->conf = &$cnf->conf;
		$this->config = &$cnf->config;
		$this->templateCode = &$templateCode;
		$this->basket = &$basket;

		$this->tt_products = &$tt_products;
		$this->tt_products_articles = &$tt_products_articles;
		$this->tt_products_cat = &$tt_products_cat;
		$this->fe_users = &$fe_users;
		$this->price = &$price;
		$this->paymentshipping = &$paymentshipping;

		$this->viewTable = &$basket->viewTable;
		$this->error_code = &$error_code;

		$this->marker = t3lib_div::makeInstance('tx_ttproducts_marker');
		$this->marker->init($this->pibase, $this->cnf, $this->basket);
	} // init


	function getOrderUid ($orderObj)	{
		$rc = FALSE;
		if (count($this->basket->itemArray))	{
			$orderUid = $orderObj->getUid();
			if (!$orderUid)	{
				$orderUid = $orderObj->getBlankUid();
			}
			$rc = $orderUid;
		}
		return $rc;
	}


	/**
	 * returns the activities in the order in which they have to be processed
	 *
	 * @param		string		  $fieldname is the field in the table you want to create a JavaScript for
	 * @return	  void
	 */
	function transformActivities ($activities)	{
		$retActivities = array();
		$codeActivities = array();
		$codeActivityArray =  Array (
			'1' =>
				'products_overview',
				'products_basket',
				'products_info',
				'products_payment',
				'products_customized_payment',
				'products_verify',
				'products_finalize',
		);

		$activityArray =  Array (
			'1' =>
			'products_redeem_gift',
			'products_clear_basket'
		);

		if (is_array($activities)) {
			foreach ($codeActivityArray as $k => $activity) {
				if ($activities[$activity]) {
					$codeActivities[$activity] = true;
				}
			}
		}

		if ($codeActivities['products_info']) {
			if($codeActivities['products_payment']) {
				$codeActivities['products_payment'] = false;
			}
			if($codeActivities['products_finalize']) {
				$codeActivities['products_finalize'] = false;
			}
		}
		if ($codeActivities['products_payment']) {
			if($codeActivities['products_finalize']) {
				$codeActivities['products_finalize'] = false;
			}
		}
		if ($codeActivities['products_basket'] && count($codeActivities)>1) {
			$codeActivities['products_basket'] = false;
		}

		if (is_array($activities)) {
			foreach ($activityArray as $k => $activity) {
				if ($activities[$activity]) {
					$retActivities[$activity] = true;
				}
			}
			$retActivities = array_merge ($retActivities, $codeActivities);
		}
		return ($retActivities);
	}


	/**
	 * processing of the payment script
	 *
	 * @param	boolean		out: $$bFinalize: determines if the order will be finalized afterwards
	 * @param	object		$basketView: tx_ttproducts_basket_view
	 * @return	string		out: content of payment script
	 */
	public function &processPayment ($orderUid, $cardRow, $pidArray, $currentPaymentActivity, &$bFinalize, &$errorMessage)	{
		global $TSFE;

		$content = '';
		include_once (PATH_BE_ttproducts.'view/class.tx_ttproducts_basket_view.php');
		$basketView = &t3lib_div::getUserObj('tx_ttproducts_basket_view');
		$basketView->init ($this->basket, $this->templateCode, $this->error_code);

		$handleScript = $TSFE->tmpl->getFileName($this->basket->basketExtra['payment.']['handleScript']);
		$handleLib = $this->basket->basketExtra['payment.']['handleLib'];
		$cnf = &t3lib_div::getUserObj('&tx_ttproducts_config');

		if ($handleScript)	{

			$content.= $this->paymentshipping->includeHandleScript($handleScript, $this->basket->basketExtra['payment.']['handleScript.'], $order, $this->conf['paymentActivity'], $bFinalize);
		} else if (strpos($handleLib,'transactor') !== FALSE && t3lib_extMgm::isLoaded($handleLib))	{
				// Payment Transactor
			require_once(t3lib_extMgm::extPath($handleLib) . 'lib/class.tx_' . $handleLib . '_api.php');
		// Get references to the concerning baskets
			$calculatedArray = $this->basket->getCalculatedArray();
			$infoObj = &t3lib_div::getUserObj('&tx_ttproducts_info');
			$addQueryString = array();
			$excludeList = '';
			$linkParams = $this->marker->getLinkParams($excludeList,$addQueryString,TRUE);
			$markerArray = $this->marker->getGlobalMarkers();
			tx_transactor_api::init($this->pibase, $this->cObj, $this->conf);
			$content = tx_transactor_api::includeHandleLib(
				$handleLib,
				$this->basket->basketExtra['payment.']['handleLib.'],
				TT_PRODUCTS_EXTkey,
				$this->basket->getItemArray(),
				$calculatedArray,
				$this->basket->recs['delivery']['note'],
				$this->conf['paymentActivity'],
				$currentPaymentActivity,
				$infoObj->infoArray,
				$pidArray,
				$linkParams,
				$this->basket->order['orderTrackingNo'],
				$orderUid,
				$cardRow,
				$bFinalize,
				$markerArray,
				$templateFilename,
				$localTemplateCode,
				$errorMessage
			);

			if ($errorMessage == '' && $content=='' && !$bFinalize && $localTemplateCode != '')	{

				$content = $basketView->getView(
					$localTemplateCode,
					'PAYMENT',
					$infoObj,
					FALSE,
					FALSE,
					TRUE,
					'###TRANSACTOR_FORM_TEMPLATE###',
					$markerArray
				);

			}
		} else if (strpos($handleLib,'paymentlib') !== FALSE && t3lib_extMgm::isLoaded($handleLib)) {

			$eInfo = tx_div2007_alpha::getExtensionInfo_fh001($handleLib);

			$paymentlibVersion = $eInfo['version'];
			$phpVersion = phpversion();
			if (isset($this->basket->basketExtra['payment.']['handleLib.']) && is_array($this->basket->basketExtra['payment.']['handleLib.']))	{
				$gatewayExtName = $this->basket->basketExtra['payment.']['handleLib.']['extName'];
			}

			if (version_compare($paymentlibVersion, '0.2.1', '>=') && version_compare($paymentlibVersion, '0.4.0', '<') && version_compare($phpVersion, '5.0.0', '>='))	{
				if ($gatewayExtName != '' && t3lib_extMgm::isLoaded($gatewayExtName))	{
					$infoObj = &t3lib_div::getUserObj('&tx_ttproducts_info');

					// Payment Library
					require_once (PATH_BE_ttproducts.'lib/class.tx_ttproducts_paymentlib.php');

					$paymentObj = t3lib_div::makeInstance('tx_ttproducts_paymentlib');
					$paymentObj->init(
						$this->pibase,
						$basketView,
						$infoObj,
						$this->urlObj
					);

					$content = $paymentObj->includeHandleLib(
						$handleLib,
						$this->basket->basketExtra['payment.']['handleLib.'],
						$bFinalize
					);
				}
			}
		}

		return $content;
	}


	/**
	 * Do all the things to be done for this activity
	 * former functions products_basket and basketView::printView
	 * Takes care of basket, address info, confirmation and gate to payment
	 * Also the 'products_...' script parameters are used here.
	 *
	 * @param		array		  CODEs for display mode
	 * @return	  void
	 */
	function doProcessing (&$codes, &$error_message) {
		global $TSFE;
		global $TYPO3_DB;

		$content = '';
		$empty = '';
		$activityArray = array();
		$bBasketEmpty = false;
		$basketView = '';
		$order = '';
		$pidTypeArray = array('PIDthanks','PIDfinalize','PIDpayment');
		$pidArray = array();
		foreach ($pidTypeArray as $pidType)	{
			if ($this->conf[$pidType])	{
				$pidArray[$pidType] = $this->conf[$pidType];
			}
		}

		$update = t3lib_div::_POST('products_update');
		$payment = t3lib_div::_POST('products_payment');
		$gpVars = t3lib_div::_GP(TT_PRODUCTS_EXTkey);
		$activityVarsArray = array(
			'clear_basket' => 'products_clear_basket',
			'customized_payment' => 'products_customized_payment',
			'finalize' => 'products_finalize',
			'info' => 'products_info',
			'overview' => 'products_overview',
			'payment' => 'products_payment',
			'redeem_gift' => 'products_redeem_gift',
			'verify' => 'products_verify'
		);

		if (!$update && !$payment && isset($gpVars) && is_array($gpVars) && isset($gpVars['activity']) && is_array($gpVars['activity']))	{
			$changedActivity = key($gpVars['activity']);
			$theActivity = $activityVarsArray[$changedActivity];
			if ($theActivity)	{
				$activityArray[$theActivity] = $gpVars['activity'][$changedActivity];
			}
		}

			// use '_x' for coordinates from Internet Explorer if button images are used
		if (t3lib_div::_GP('products_redeem_gift') || t3lib_div::_GP('products_redeem_gift_x'))    {
		 	$activityArray['products_redeem_gift'] = true;
		}
		if (t3lib_div::_GP('products_clear_basket') || t3lib_div::_GP('products_clear_basket_x'))    {
			$activityArray['products_clear_basket'] = true;
		}
		if (t3lib_div::_GP('products_overview') || t3lib_div::_GP('products_overview_x'))    {
			$activityArray['products_overview'] = true;
		}
		if (!$update) {
			if (t3lib_div::_GP('products_payment') || t3lib_div::_GP('products_payment_x'))    {
				$activityArray['products_payment'] = TRUE;
			} else if (t3lib_div::_GP('products_info') || t3lib_div::_GP('products_info_x'))    {
				$activityArray['products_info'] = TRUE;
			}
		}
		if (t3lib_div::_GP('products_customized_payment') || t3lib_div::_GP('products_customized_payment_x'))    {
			$activityArray['products_customized_payment'] = true;
		}
		if (t3lib_div::_GP('products_verify') || t3lib_div::_GP('products_verify_x'))    {
			$activityArray['products_verify'] = TRUE;
		}
		if (t3lib_div::_GP('products_finalize') || t3lib_div::_GP('products_finalize_x'))    {
			$activityArray['products_finalize'] = true;
		}
		$codeActivityArray=array();
		$bBasketCode = false;
		if (is_array($codes)) {
			foreach ($codes as $k => $code) {
				if ($code=='BASKET')	{
					$codeActivityArray['products_basket']=true;
					$bBasketCode = true;
				} elseif ($code=='INFO' && !($activityArray['products_payment'] || $activityArray['products_verify'] || $activityArray['products_finalize']))	{
					$codeActivityArray['products_info']=true;
					$bBasketCode = true;
				} elseif ($code=='OVERVIEW') {
					$codeActivityArray['products_overview']=true;
				} elseif ($code=='PAYMENT' && !($activityArray['products_finalize']))	{
					$codeActivityArray['products_payment']=true;
				} elseif ($code=='FINALIZE')	{
					$codeActivityArray['products_finalize']=true;
				}
			}
		}

		if ($bBasketCode)	{
			$activityArray = array_merge ($activityArray, $codeActivityArray);
			$this->activityArray = $this->transformActivities($activityArray);
		} else {
			// only the code activities if there is no code BASKET or INFO set
			$this->activityArray = $codeActivityArray;
		}

		$infoObj = &t3lib_div::getUserObj('&tx_ttproducts_info');
		$infoObj->init($this->pibase, $this->cnf, $this->basket->recs, $this->fe_users, $this->paymentshipping, $activityArray['products_payment']);
		if ($infoObj->checkRequired('billing')=='')	{
			$infoObj->mapPersonIntoDelivery();
		}

		if (count($this->basket->itemArray) && count($this->activityArray))	{	// If there is content in the shopping basket, we are going display some basket code
				// prepare action
			$basket_tmpl = '';

			if (count($this->activityArray)) {
				// $this->basket->getCalculatedBasket();  // all the basket calculation is done in this function once and not multiple times here
				$mainMarkerArray=array();
				$mainMarkerArray['###EXTERNAL_COBJECT###'] = $this->pibase->externalCObject.'';  // adding extra preprocessing CObject
				$bFinalize = FALSE; // no finalization must be called.

				if ($this->activityArray['products_info'] || $this->activityArray['products_payment'] || $this->activityArray['products_customized_payment'] || $this->activityArray['products_verify'] || $this->activityArray['products_finalize'])	{
					// get credit card info
					include_once (PATH_BE_ttproducts.'model/class.tx_ttproducts_card.php');
					$cardObj = t3lib_div::makeInstance('tx_ttproducts_card');
					$cardObj->init(
						$this->pibase,
						$this->cnf,
						$this->basket->recs,
						$this->basket->basketExtra['payment.']['creditcards']
					);
					$cardObj->getItemMarkerArray($mainMarkerArray);
					$cardUid = $cardObj->getUid();
					$cardRow = $cardObj->get($cardUid);

					// get bank account info
					include_once (PATH_BE_ttproducts.'model/class.tx_ttproducts_account.php');
					$account = t3lib_div::makeInstance('tx_ttproducts_account');
					$account->init(
						$this->pibase,
						$this->cnf,
						$this->basket->recs,
						$this->basket->basketExtra['payment.']['accounts']
					);
					$account->getItemMarkerArray($mainMarkerArray);
				}

				foreach ($this->activityArray as $activity => $value) {
					if ($value) {
						$currentPaymentActivity = array_search($activity, $activityVarsArray);

							// perform action
						switch($activity)	{
							case 'products_clear_basket':
								// Empties the shopping basket!
								$this->basket->clearBasket(TRUE);
								$bBasketEmpty = TRUE;
							break;
							case 'products_basket':
								if (count($this->activityArray) == 1) {
									$basket_tmpl = 'BASKET_TEMPLATE';
								}
							break;
							case 'products_overview':
								$this->pibase->load_noLinkExtCobj();	// TODO
								$basket_tmpl = 'BASKET_OVERVIEW_TEMPLATE';
							break;
							case 'products_redeem_gift': 	// this shall never be the only activity
								if (trim($TSFE->fe_user->user['username']) == '') {
									$basket_tmpl = 'BASKET_TEMPLATE_NOT_LOGGED_IN';
								} else {
									$uniqueId = t3lib_div::trimExplode ('-', $this->basket->recs['tt_products']['giftcode'], true);
									$query='uid=\''.intval($uniqueId[0]).'\' AND crdate=\''.$uniqueId[1].'\''.' AND NOT deleted' ;
									$giftRes = $TYPO3_DB->exec_SELECTquery('*', 'tt_products_gifts', $query);
									$row = $TYPO3_DB->sql_fetch_assoc($giftRes);
									$pricefactor = doubleval($this->conf['creditpoints.']['pricefactor']);

									if ($row && $pricefactor > 0) {
										$money = $row['amount'];
										$uid = $row['uid'];
										$fieldsArray = array();
										$fieldsArray['deleted']=1;

											// Delete the gift record
										$TYPO3_DB->exec_UPDATEquery('tt_products_gifts', 'uid='.intval($uid), $fieldsArray);
										$creditpoints = $money / $pricefactor;
										tx_ttproducts_creditpoints_div::addCreditPoints($TSFE->fe_user->user['username'], $creditpoints);
										$cpArray = $TSFE->fe_user->getKey('ses','cp');
										$cpArray['gift']['amount'] += $creditpoints;
										$TSFE->fe_user->setKey('ses','cp',$cpArray);
									}
								}
							break;
							case 'products_info':
								$this->pibase->load_noLinkExtCobj(); // TODO
								$basket_tmpl = 'BASKET_INFO_TEMPLATE';
								// }
							break;
							case 'products_payment':
								$this->pibase->load_noLinkExtCobj();	// TODO
								$pidagb = intval($this->conf['PIDagb']);

								$checkRequired = $infoObj->checkRequired();
								$checkAllowed = $infoObj->checkAllowed();
								if ($this->paymentshipping->useCreditcard ())	{
									$cardRequired = $cardObj->checkRequired();
								}
								if ($this->paymentshipping->useAccount())	{
									$accountRequired = $account->checkRequired();
								}

								if ($this->conf['paymentActivity'] == 'payment' || $this->conf['paymentActivity'] == 'verify')	{

									if (!is_object($order))	{
										include_once (PATH_BE_ttproducts.'model/class.tx_ttproducts_order.php');

											// order
										$order = &t3lib_div::getUserObj('tx_ttproducts_order');
										$order->init(
											$this->pibase,
											$this->cnf,
											$this->tt_products,
											$this->tt_products_articles,
											$this->tt_products_cat,
											$this->basket,
											$this->conf['useArticles']
										);
									}
									$handleLib = $this->paymentshipping->getHandleLib('request');

									if (strpos($handleLib,'transactor') !== FALSE)	{
										// Payment Transactor
										require_once(t3lib_extMgm::extPath($handleLib) . 'lib/class.tx_' . $handleLib . '_api.php');
										tx_transactor_api::init($this->pibase, $this->cObj, $this->conf);
										$referenceId = tx_transactor_api::getReferenceUid(
											$handleLib,
											$this->basket->basketExtra['payment.']['handleLib.'],
											TT_PRODUCTS_EXTkey,
											$orderUid
										);
										$addQueryString = array();
										$excludeList = '';
										$linkParams = $this->marker->getLinkParams($excludeList,$addQueryString,TRUE);
										$paymentErrorMsg = tx_transactor_api::checkRequired(
											$referenceId,
											$this->basket->basketExtra['payment.']['handleLib'],
											$this->basket->basketExtra['payment.']['handleLib.'],
											TT_PRODUCTS_EXTkey,
											$this->basket->getCalculatedArray(),
											$this->conf['paymentActivity'],
											$pidArray,
											$linkParams,
											$this->basket->order['orderTrackingNo'],
											$orderUid,
											$cardRow
										);
									} else if (strpos($handleLib,'paymentlib') !== FALSE)	{
										require_once (PATH_BE_ttproducts.'lib/class.tx_ttproducts_paymentlib.php');
										$paymentlib = t3lib_div::makeInstance('tx_ttproducts_paymentlib');
										$paymentlib->init($this->pibase, $this->cnf, $this->basket, $basketView, $this->price, $order, $infoObj, $card, $account);
										$paymentErrorMsg = $paymentlib->checkRequired($this->basket->basketExtra['payment.']['handleLib.']);
									}
								}

								if ($checkRequired == '' && $checkAllowed == '' && $cardRequired == '' && $accountRequired == '' && $paymentErrorMsg == '' &&
									(empty($pidagb) || $_REQUEST['recs']['personinfo']['agb'] || $this->pibase->piVars['agb'] || $infoObj->infoArray['billing']['agb'])) {

									// basket view
									if (!is_object($basketView))	{
										include_once (PATH_BE_ttproducts.'view/class.tx_ttproducts_basket_view.php');
										$basketView = &t3lib_div::getUserObj('tx_ttproducts_basket_view');
										$basketView->init ($this->basket, $this->templateCode, $this->error_code);
									}
									if ($this->conf['paymentActivity'] == 'payment' || $this->conf['paymentActivity'] == 'verify')	{

										$orderUid = $this->getOrderUid($order);
										$mainMarkerArray['###MESSAGE_PAYMENT_SCRIPT###'] =
											$this->processPayment(
												$orderUid,
												$cardRow,
												$pidArray,
												$currentPaymentActivity,
												$bFinalize,
												$errorMessage
											);

										if ($errorMessage != '')	{
											$mainMarkerArray['###MESSAGE_PAYMENT_SCRIPT###'] = $errorMessage;
										}
									} else {
										$mainMarkerArray['###MESSAGE_PAYMENT_SCRIPT###'] = '';
									}
									$content .= $basketView->getView($empty, 'PAYMENT', $infoObj, $this->activityArray['products_info'], false, true, '###BASKET_PAYMENT_TEMPLATE###', $mainMarkerArray);
								} else {	// If not all required info-fields are filled in, this is shown instead:
									$content .= $this->pibase->cObj->getSubpart($this->templateCode,$this->marker->spMarker('###BASKET_REQUIRED_INFO_MISSING###'));
									$label = '';
									$addQueryString = array();
									$overwriteMarkerArray = array();
									if ($pidagb && !$_REQUEST['recs']['personinfo']['agb'] &&  !$infoObj->infoArray['billing']['agb'] && !t3lib_div::_GET('products_payment')) {
										 // so AGB has not been accepted
										$label = $this->pibase->pi_getLL('accept_AGB');
										$addQueryString['agb']=0;
									} else if ($cardRequired)	{
										$label = '*'.$this->pibase->pi_getLL($cardObj->tablename.'.'.$cardRequired).'*';
									} else if ($accountRequired)	{
										$label = '*'.$this->pibase->pi_getLL($account->tablename).': '.$this->pibase->pi_getLL($account->tablename.'.'.$accountRequired).'*';
									} else if ($paymentErrorMsg)	{
										$label = $paymentErrorMsg;
									} else {
										if (t3lib_extMgm::isLoaded('sr_feuser_register')) {
											$check = ($checkRequired ? $checkRequired: $checkAllowed);
											$label = $TSFE->sL('LLL:EXT:sr_feuser_register/pi1/locallang.php:missing_'.$check);
											$editPID = $TSFE->tmpl->setup['plugin.']['tx_srfeuserregister_pi1.']['editPID'];
											if ($TSFE->loginUser && $editPID) {
												$addParams = array ('products_payment' => 1);
												$addParams = $this->marker->getLinkParams('',$addParams,true);
												$srfeuserBackUrl = $this->pibase->pi_getPageLink($TSFE->id,'',$addParams);
												$srfeuserParams = array('tx_srfeuserregister_pi1[backURL]' => $srfeuserBackUrl);
												$addParams = $this->marker->getLinkParams('',$srfeuserParams,true);
												$overwriteMarkerArray['###FORM_URL_INFO###'] = $this->pibase->pi_getPageLink($editPID,'',$addParams);
											}
										} else {
											$label = 'field: '.$checkRequired;
										}
									}
									$markerArray = $this->marker->addURLMarkers(0, array(), $addQueryString);
									$markerArray['###ERROR_DETAILS###'] = $label;
									$markerArray = array_merge($markerArray,$overwriteMarkerArray);

									$content = $this->pibase->cObj->substituteMarkerArray($content, $markerArray);
								}
							break;
							// a special step after payment and before finalization needed for some payment systems
							case 'products_customized_payment':
							case 'products_verify':
								if ($this->conf['paymentActivity']=='verify' || $this->conf['paymentActivity']=='customized' /* deprecated */)	{
									if (!is_object($order))	{
										include_once (PATH_BE_ttproducts.'model/class.tx_ttproducts_order.php');

										$order = &t3lib_div::getUserObj('tx_ttproducts_order');
										$order->init(
											$this->pibase,
											$this->cnf,
											$this->tt_products,
											$this->tt_products_articles,
											$this->tt_products_cat,
											$this->basket,
											$this->conf['useArticles']
										);
									}
									// basket view
									if (!is_object($basketView))	{
										include_once (PATH_BE_ttproducts.'view/class.tx_ttproducts_basket_view.php');
										$basketView = &t3lib_div::getUserObj('tx_ttproducts_basket_view');
										$basketView->init ($this->basket, $this->templateCode, $this->error_code);
									}
									$paymentContent = '';
									$orderUid = $this->getOrderUid($order);
									$mainMarkerArray['###MESSAGE_PAYMENT_SCRIPT###'] =
										$this->processPayment(
											$orderUid,
											$cardRow,
											$pidArray,
											$currentPaymentActivity,
											$bFinalize,
											$errorMessage
										);

									if ($errorMessage != '')	{
										$mainMarkerArray['###MESSAGE_PAYMENT_SCRIPT###'] = $errorMessage;
									}

									if (!$bFinalize)	{
										$content = $basketView->getView($empty, 'PAYMENT', $infoObj, $this->activityArray['products_info'], false, true, '###BASKET_PAYMENT_TEMPLATE###', $mainMarkerArray);
									}
								} else {
									$mainMarkerArray['###MESSAGE_PAYMENT_SCRIPT###'] = '';
								}
							break;
							case 'products_finalize':

								$handleLib = $this->paymentshipping->getHandleLib('request');
								if ($handleLib == '')	{
									$handleLib = $this->paymentshipping->getHandleLib('form');
								}

								if ($handleLib != '') {
									if (!is_object($order)) {
										include_once (PATH_BE_ttproducts.'model/class.tx_ttproducts_order.php');
										// order
										$order = &t3lib_div::getUserObj('tx_ttproducts_order');
										$order->init(
											$this->pibase,
											$this->cnf,
											$this->tt_products,
											$this->tt_products_articles,
											$this->tt_products_cat,
											$this->basket,
											$this->conf['useArticles']
										);
									}
									$orderUid = $this->getOrderUid($order);
									$rc = $this->processPayment(
										$orderUid,
										$cardRow,
										$pidArray,
										$currentPaymentActivity,
										$bFinalize,
										$errorMessage
									);

									if($bFinalize == FALSE ){
										$label = $errorMessage;
										$addQueryString = array();
										$markerArray = $this->marker->addURLMarkers(0, $markerArray, $addQueryString);
										$content = $this->pibase->cObj->getSubpart($this->templateCode,$this->marker->spMarker('###BASKET_REQUIRED_INFO_MISSING###'));
										$markerArray['###ERROR_DETAILS###'] = $label;
										$content = $this->pibase->cObj->substituteMarkerArray($content, $markerArray);
										$bFinalize = false;
									} else {
										$bFinalize = true;
									}
								} else {
									$bFinalize = true;
								}
							break;
							default:
								// nothing yet
								$activity = 'unknown';
							break;
						} // switch
					}	// if ($value)
					if ($bBasketEmpty)	{
						break;
					}

					if ($this->basket->checkMinPrice)	{
						$basketConf = $this->cnf->getBasketConf('minPrice'); // check the basket limits
						$bNeedsMinCheck = in_array($activity, array('products_info','products_payment', 'products_customized_payment', 'products_verify', 'products_finalize', 'unknown'));
						if ($bNeedsMinCheck && $basketConf['type'] == 'price')	{
							$value = $this->basket->calculatedArray['priceTax'][$basketConf['collect']];
							if (isset($value) && isset($basketConf['collect']) && $value < doubleval($basketConf['value']))	{
								$basket_tmpl = 'BASKET_TEMPLATE_MINPRICE_ERROR';
								$bFinalize = false;
							}
						}
					}

						// in case the basket template has been set
					if ($basket_tmpl) {
						// basket view
						if (!is_object($basketView))	{
							include_once (PATH_BE_ttproducts.'view/class.tx_ttproducts_basket_view.php');
							$basketView = &t3lib_div::getUserObj('tx_ttproducts_basket_view');
							$basketView->init ($this->basket, $this->templateCode, $this->error_code);
						}
						$content .= $basketView->getView($empty, 'BASKET', $infoObj, $this->activityArray['products_info'], FALSE, TRUE, '###'.$basket_tmpl.'###',$mainMarkerArray);
						$bFinalize = FALSE;
						// stop here as soon as the first basket content has been drawn
						break; // foreach
					}
				} // foreach ($this->activityArray as $activity=>$value)

					// finalization at the end so that after every activity this can be called
				if ($bFinalize)	{
					$checkRequired = $infoObj->checkRequired();
					$checkAllowed = $infoObj->checkAllowed();

					if ($checkRequired == '' && $checkAllowed == '')	{
						$this->pibase->load_noLinkExtCobj();	// TODO
						$handleScript = $TSFE->tmpl->getFileName($this->basket->basketExtra['payment.']['handleScript']);
						if (!is_object($order))	{
							include_once (PATH_BE_ttproducts.'model/class.tx_ttproducts_order.php');

								// order
							$order = &t3lib_div::getUserObj('tx_ttproducts_order');
							$order->init(
								$this->pibase,
								$this->cnf,
								$this->tt_products,
								$this->tt_products_articles,
								$this->tt_products_cat,
								$this->basket,
								$this->conf['useArticles']
							);
						}
						$orderUid = $this->getOrderUid($order);

						if (!is_object($basketView))	{
							include_once (PATH_BE_ttproducts.'view/class.tx_ttproducts_basket_view.php');
							$basketView = &t3lib_div::getUserObj('tx_ttproducts_basket_view');
							$basketView->init ($this->basket, $this->templateCode, $this->error_code);
						}

						if ($this->conf['paymentActivity'] == 'finalize') {
							$mainMarkerArray['###MESSAGE_PAYMENT_SCRIPT###'] =
								$this->processPayment(
									$orderUid,
									$cardRow,
									$pidArray,
									'finalize',
									$bFinalize,
									$errorMessage
								);

							if ($errorMessage != '')	{
								$mainMarkerArray['###MESSAGE_PAYMENT_SCRIPT###'] = $errorMessage;
							}
						} else {
							$mainMarkerArray['###MESSAGE_PAYMENT_SCRIPT###'] = '';
						}

						// get the orderconfirmation template as html email and the thanks template as thanks page
						$tmpl = 'BASKET_ORDERCONFIRMATION_TEMPLATE';
						$orderConfirmationHTML = $basketView->getView($empty, 'BASKET', $infoObj, false, false, true, '###'.$tmpl.'###', $mainMarkerArray);

						include_once (PATH_BE_ttproducts.'control/class.tx_ttproducts_activity_finalize.php');
							// order finalization
						$activityFinalize = t3lib_div::makeInstance('tx_ttproducts_activity_finalize');
						$activityFinalize->init($this->pibase, $this->cnf, $this->basket, $this->tt_products, $this->tt_products_cat, $this->fe_users, $order, $mainMarkerArray);
						$activityFinalize->doProcessing(
							$this->templateCode,
							$basketView,
							$this->viewTable,
							$this->price,
							$orderUid,
							$orderConfirmationHTML,
							$error_message,
							$infoObj,
							$mainMarkerArray
						);
								// Important: 	 MUST come after the call of prodObj->getView, because this function, getView, calculates the order! And that information is used in the finalize-function
						$contentTmp = $orderConfirmationHTML;

						if ($this->conf['PIDthanks'] > 0) {
							$tmpl = 'BASKET_ORDERTHANKS_TEMPLATE';
							$contentTmpThanks = $basketView->getView($empty, 'BASKET', $infoObj, false, false, true, '###'.$tmpl.'###', $mainMarkerArray);
							if ($contentTmpThanks != '')	{
								$contentTmp = $contentTmpThanks;
							}
						}
						$content .= $contentTmp;
						$content .= $basketView->getView($empty, 'BASKET', $infoObj, false, false, true, '###BASKET_ORDERCONFIRMATION_NOSAVE_TEMPLATE###');

						// Empties the shopping basket!
						$this->basket->clearBasket();
					} else {	// If not all required info-fields are filled in, this is shown instead:
						$content .= $this->pibase->cObj->getSubpart($this->templateCode,$this->marker->spMarker('###BASKET_REQUIRED_INFO_MISSING###'));
						$content = $this->pibase->cObj->substituteMarkerArray($content, $this->marker->addURLMarkers(0, array()));
					}
				}
			} else { // if (count($this->activityArray))
				// nothing. no BASKET code or similar thing
			}
		} else { // if (count($this->basket->basketExt))
			$bBasketEmpty = true;
		}

		$basketMarkerArray = array();
		if (!is_object($basketView))	{
			include_once (PATH_BE_ttproducts.'view/class.tx_ttproducts_basket_view.php');
			$basketView = &t3lib_div::getUserObj('tx_ttproducts_basket_view');
			$basketView->init($this->basket, $this->templateCode, $this->error_code);
		}

		if ($bBasketEmpty)	{
			if ($this->activityArray['products_overview']) {
				$this->pibase->load_noLinkExtCobj();	//
				$content .= $this->pibase->cObj->getSubpart($this->templateCode,$this->marker->spMarker('###BASKET_OVERVIEW_EMPTY###'));
			} else if ($this->activityArray['products_basket'] || $this->activityArray['products_info'] || $this->activityArray['products_payment']) {
				$content .= $this->pibase->cObj->getSubpart($this->templateCode,$this->marker->spMarker('###BASKET_TEMPLATE_EMPTY###'));
			}
		}
		$basketMarkerArray = $basketView->getMarkerArray();
		$markerArray = $basketMarkerArray;
		$markerArray['###EXTERNAL_COBJECT###'] = $this->pibase->externalCObject;	// adding extra preprocessing CObject
		$content = $this->pibase->cObj->substituteMarkerArray($content, $markerArray);

		return $content;
	} //
}


if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/tt_products/control/class.tx_ttproducts_control.php']) {
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/tt_products/control/class.tx_ttproducts_control.php']);
}


?>
